@@ -1,57 +1,61 @@
+use clap::Parser;
 use colored::control::set_override;
 use core::panic;
 use rayon::prelude::*;
 use regex::Regex;
-use ropr::{binary::Binary, formatter::ColourFormatter, gadgets::Disassembly};
+use ropr::{binary::Binary, disassembler::Disassembly, formatter::ColourFormatter};
 use std::{
 	collections::HashSet,
+	error::Error,
 	io::{stdout, BufWriter, Write},
 	path::PathBuf,
 	time::Instant,
 };
-use structopt::StructOpt;
 
-#[derive(StructOpt)]
-#[structopt(name = "ropr")]
+#[derive(Parser)]
+#[clap(name = "ropr")]
 struct Opt {
-	#[structopt(short = "n", long)]
+	#[clap(short = 'n', long)]
 	noisy: bool,
 
-	#[structopt(short = "c", long)]
+	#[clap(short = 'c', long)]
 	colour: Option<bool>,
 
-	#[structopt(short = "r", long)]
+	#[clap(short = 'r', long)]
 	norop: bool,
 
-	#[structopt(short = "s", long)]
+	#[clap(short = 's', long)]
 	nosys: bool,
 
-	#[structopt(short = "j", long)]
+	#[clap(short = 'j', long)]
 	nojop: bool,
 
-	#[structopt(short = "p", long)]
+	#[clap(short = 'p', long)]
 	stack_pivot: bool,
 
-	#[structopt(short = "b", long)]
+	#[clap(short = 'b', long)]
 	base_pivot: bool,
 
-	#[structopt(short, long, default_value = "6")]
+	#[clap(short, long, default_value = "6")]
 	max_instr: u8,
 
-	#[structopt(short = "R", long)]
-	regex: Option<String>,
+	#[clap(short = 'R', long)]
+	regex: Vec<String>,
+
+	#[clap(long)]
+	raw: Option<bool>,
 
 	binary: PathBuf,
 }
 
-fn main() {
+fn main() -> Result<(), Box<dyn Error>> {
 	let start = Instant::now();
 
-	let opts = Opt::from_args();
+	let opts = Opt::parse();
 
 	let b = opts.binary;
-	let b = Binary::new(&b).unwrap();
-	let sections = b.sections().unwrap();
+	let b = Binary::new(&b)?;
+	let sections = b.sections(opts.raw)?;
 
 	let noisy = opts.noisy;
 	let colour = opts.colour;
@@ -66,15 +70,19 @@ fn main() {
 		panic!("Max instructions must be >0");
 	}
 
-	let regex = opts.regex.map(|r| Regex::new(&r).unwrap());
+	let regices = opts
+		.regex
+		.into_iter()
+		.map(|r| Regex::new(&r))
+		.collect::<Result<Vec<_>, _>>()?;
 
 	let gadgets = sections
 		.iter()
 		.filter_map(Disassembly::new)
 		.flat_map(|dis| {
-			let tails = dis.tails(rop, sys, jop, noisy).collect::<Vec<_>>();
-			tails
+			(0..dis.bytes().len())
 				.into_par_iter()
+				.filter(|offset| dis.is_tail_at(*offset, rop, sys, jop, noisy))
 				.flat_map_iter(|tail| {
 					dis.gadgets_from_tail(tail, max_instructions_per_gadget, noisy)
 				})
@@ -89,10 +97,7 @@ fn main() {
 			g.format_instruction(&mut formatted);
 			(g, formatted)
 		})
-		.filter(|(_, formatted)| match &regex {
-			Some(r) => r.is_match(formatted),
-			None => true,
-		})
+		.filter(|(_, formatted)| regices.iter().all(|r| r.is_match(formatted)))
 		.filter(|(g, _)| !stack_pivot | g.is_stack_pivot())
 		.filter(|(g, _)| !base_pivot | g.is_base_pivot())
 		.collect::<Vec<_>>();
@@ -117,11 +122,13 @@ fn main() {
 		writeln!(stdout, "{}", output).unwrap();
 	}
 
-	stdout.flush().unwrap();
+	drop(stdout);
 
 	eprintln!(
 		"\n==> Found {} gadgets in {:.3} seconds",
 		gadget_count,
 		elapsed.as_secs_f32()
 	);
+
+	Ok(())
 }
